package de.gjlay.ebook;

import java.io.*;
import java.util.*;

import de.gjlay.tools.*;

public class Split
{
	public static final String USAGE =
	"Usage:\n"
	+"Encode [-size <size>] <infile> [<outfile>]\n"
	+"<size> is the size of output file(s).\n"
	+"<size> may end with 'k' or 'M', k means kByte, M means MByte.\n"
	+"<infile> input file name.\n"
	+"<outfile> output file name, defaults to <infile>-*.\n"
	+"If <outfile> contains a '@', the @ is replaced by output file number.\n"
	+"Counting starts at 0.\n"
	;

	public static void main (String[] args)
		throws Exception
		{
			System.out.println ("This is Split");

			try
			{
				Splitter enc = new Splitter (args);
				enc.run();
			}
			catch (ArgumentException ex)
			{
				ex.printStackTrace();
				System.out.println (USAGE);
			}
		}
}

class Splitter
{
	final static String[] SWITCHES = {};
	final static String[] OPTIONS = {"-size"};

	final static int BUFSIZE = 1000;
	static byte[] buf;

	int fileSize = 0;

	String[] args;
	String inName, outName;

	Splitter (String[] args)
		{
			this.args = args;
		}

	void run()
		throws Exception
		{
			evalArguments (args);

			buf = new byte[fileSize];

			int nread, i;
			FileInputStream inStream = new FileInputStream (inName);

			for (i=0; (nread = inStream.read (buf)) > 0; i++)
			{
				String outName = getOutName (i);
				FileOutputStream outStream;

				System.out.println ("Writing " + nread + " to " + outName);
				outStream = new FileOutputStream (outName, false);
				outStream.write (buf, 0, nread);
				outStream.close();
			}

			System.out.println ("Written " + i + " files.");

			inStream.close();
		}

	String getOutName (int index)
		{
			return StringTools.replace (outName, "@", ""+index);
		}

	void evalArguments (String[] args)
		throws ArgumentException
		{
			Arguments eval = new Arguments (SWITCHES, OPTIONS);

			eval.parse (args, true);

			if (eval.hasOption ("-size"))
			{
				fileSize = getFileSize (eval.getOption ("-size"));
				System.out.println ("Size = " + fileSize);
			}

			if (eval.getArgumentCount() < 1)
				throw new ArgumentException ("No input file name specified");

			inName = eval.getArgument (0);

			if (eval.getArgumentCount() >= 2)
				outName = eval.getArgument (1);
			else
				outName = inName+"-@";

			if (outName.indexOf ("@") < 0)
				throw new ArgumentException ("Output file name needs @");
		}

	int getFileSize (String str)
		throws ArgumentException
		{
			int size, mul = 1;

			str = str.trim();

			if (str.endsWith ("k"))				mul = 1024;
			if (str.endsWith ("M"))				mul = 1024*1024;

			if (mul != 1)
				str = str.substring (0, str.length()-1);

			size = StringTools.toInt (str, -1);

			if (-1 == size)
				throw new ArgumentException ("Invalid argument to -msize");

			return size * mul;
		}
}
