#ifndef _TASTER_H_
#define _TASTER_H_

// One of 1...4
#define NUM_TASTER 3

#include <inttypes.h>

typedef struct 
{
    // internal use
    uint8_t delay;
    uint8_t old;

    // the mode, one of KM_SHORT, KM_LONG, KM_REPREAT
    uint8_t mode;
} taste_t;

// set tasten[n].mode to configure/change key n behaviour
// tasten[0] is an unised dummy, therefore we need a +1
extern taste_t tasten[NUM_TASTER+1];

/*
  get_taster() is to be called in timer intervals of 10ms (or about that)
  The preferred way is to call this functon from an ISR
  and not from the application code that wants to get key
  press events. If taster is 0 (equals NO_KEY) then get_taster
  uptates this variable to reflect a key press (if there is one).
  
  Therefore, as soon as the application sees a nen-zero value in
  taster, it should backup the value in a local variable and
  clear taster.

  Eventually, taster is something like a fifo that hold yust
  one entry.

  By calling get_taster in relatively long (with respect to
  MCU clock) intervals of 10ms we automatically get the keys
  debounced.
*/  
extern void get_taster (uint8_t);
extern volatile uint8_t taster;

// These values control the keys' behaviour
// The unit is the time inverval get_taster is called,
// e.g. if get_taster is called every 10ms,
// then a 10 means 100ms
#define KEYREPEAT_DELAY 50
#define KEYREPEAT       20
#define KEYDELAY_LONG   100

enum
{
    // The key gives an event every time it is pressed
    // the event is just the number of the key, i.e.
    // a short press of tasten[1] in this mode will set
    // taster to 1.
    KM_SHORT,

    // The key gives a short or long key press event
    // dependent on the duration.
    // If it is pressed at least KEYDELAY_LONG, then it will
    // give a long event, e.g. KEY_DOWN_L (same as KEY_DOWN + KEY_LONG)
    // for KEY_DOWN.
    // If it is released before KEYDELAY_LONG is over, then it will
    // give a short event, e.g.KEY_DOWN for KEY_DOWN.
    KM_LONG,

    // Pressing the key gives a short event.
    // If the key is hold down then after KEYREPEAT_DELAY
    // there will be generated new short events that occur in
    // intervals of KEYREPEAT.
    KM_REPEAT
};

#define KEY_LONG NUM_TASTER

// Just add/remove keys as needed
enum 
{
    NO_KEY = 0,
    KEY_UP,
    KEY_MITTE,
    KEY_DOWN,
 
    KEY_UP_L    = KEY_LONG + KEY_UP,
    KEY_MITTE_L = KEY_LONG + KEY_MITTE,
    KEY_DOWN_L  = KEY_LONG + KEY_DOWN
 
};

#endif /* _TASTER_H_ */
