#include <avr/io.h>

#include "ports.h"
#include "timer.h"

countdown_t volatile count;

static void down (uint8_t, uint8_t volatile *);

// wait t*10ms to (t+1)*10ms
void wait_10ms (const uint8_t t)
{
    count.ms10.wait_10ms = 1+t;
    while (count.ms10.wait_10ms)
        ;
}

// just a helper to job_timer_10ms
void down (uint8_t i, uint8_t volatile * cd)
{
    do
    {
        uint8_t val = *(--cd) -1;
  
        if (val != 0xff)
            *cd = val;
    } while (--i);
}

// count down a portion of count (every member as uint8_t)
#define DOWN(x)                                             \
    do                                                      \
    {                                                       \
        if (!sizeof (x))                                    \
            return;                                         \
        down (sizeof (x), (uint8_t *) & x + sizeof (x));    \
    } while (0)

// to be called every 10ms
void job_timer_10ms()
{
    DOWN (count.ms10);

    if (count.ms10.timer_1s)
        return;
    count.ms10.timer_1s = 100;
  
    DOWN (count.sec);
}  

#if !defined (TCNT1H)
#error Cannot use Timer1 this way!
#endif // TCNT1H

void init_timer1()
{
    // Timer1 is counter, no PWM
    TCCR1A = 0;

    // Clear Timer on Compare Match (CTC, Mode #4)
    // Full F_CPU (Prescale = 1)
#if defined (CTC1) && !defined (WGM12)
    TCCR1B = (1 << CTC1)  | (1 << CS10);
#elif !defined (CTC1) && defined (WGM12)
    TCCR1B = (1 << WGM12) | (1 << CS10);
#else    
#error Cannot initialize Timer1 this way!
#endif

    // set OutputCompare to get desired Timer1 frequency
    OCR1A = (uint16_t) ((uint32_t) F_CPU / IRQS_PER_SECOND -1);

    // enable OutputCompareA Interrupt for Timer1
#if defined (TIMSK1)
    TIMSK1 |= (1 << OCIE1A);
#elif defined (TIMSK)
    TIMSK  |= (1 << OCIE1A);
#else	 
#error Cannot initialize Timer1 IRQs this way!
#endif
}
