#ifndef _TIMER_H_
#define _TIMER_H_

#include <inttypes.h>

// Make sure that
//
//    F_CPU / IRQS_PER_SECOND -1
//
// yields a sensible 16-bit value.
// The smaller the remainder of the division, the exacter
// the timer will operate.
#define IRQS_PER_SECOND    10000
#define IRQS_PER_10MS   (IRQS_PER_SECOND / 100) 

// Just add new members as needed
// all members are 8-bit countdown timers.
// They will decrement to 0 and then stop.
typedef struct
{
    struct
    {
        // Internal use, do not change or remove.
        // Used to track seconds and know when to
        // decrement the elements of .sec
        uint8_t timer_1s;

        // used by wait_10ms
        uint8_t wait_10ms;

        // Keeps track of key strokes when displaying
        // book/chapter/verse information
        uint8_t key_timer;

        // Slows down display of new letters/display shifts per sec
        uint8_t ebook_delay;
    } ms10;

    struct
    {
        // config() will exit and restore ebook.eeprom if this
        // countdown runs dry
        uint8_t config_timeout;
    } sec;
} countdown_t;

// This structure holds several countdown timers.
// If a timer is 0 its value is not changed
// otherwise, its value is decremented
// - every 10ms   (members of count.ms10)
// - every second (members of count.sec)
extern volatile countdown_t count;

// Waits at about 10ms
extern void wait_10ms (const uint8_t);

// Must be called once every 10ms to count down the members of count.
extern void job_timer_10ms();

// Initialize Timer1 to perform IRQS_PER_SECOND IRQs per second.
extern void init_timer1 (void);

#endif /* _TIMER_H_ */
